﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows.Input;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI;
using Windows.UI.Popups;
using Windows.UI.Xaml.Media.Imaging;
using FaceIdentify.Models;
using FaceIdentify.Services;
using FaceIdentify.Views;
using Template10.Mvvm;

namespace FaceIdentify.ViewModels
{
    /// <summary>
    ///     인식 페이지
    /// </summary>
    public class IdentifyPageViewModel : ViewModelBase
    {
        private BitmapSource _bitmapSource;
        private IList<PersonModel> _identifyFaceList;

        /// <summary>
        ///     기본 생성자
        /// </summary>
        public IdentifyPageViewModel()
        {
            if (DesignMode.DesignModeEnabled)
            {
                //디자인 타임 데이터
            }
            else
            {
                Init();
            }
        }

        /// <summary>
        ///     비트맵 소스
        /// </summary>
        public BitmapSource BitmapSource
        {
            get { return _bitmapSource; }
            set { Set(ref _bitmapSource, value); }
        }

        /// <summary>
        ///     인식한 얼굴 목록
        /// </summary>
        public IList<PersonModel> IdentifyFaceList
        {
            get { return _identifyFaceList; }
            set { Set(ref _identifyFaceList, value); }
        }

        /// <summary>
        ///     사진 열기 커맨드
        /// </summary>
        public ICommand OpenPictureCommand { get; set; }

        /// <summary>
        ///     초기화
        /// </summary>
        private void Init()
        {
            IdentifyFaceList = new ObservableCollection<PersonModel>();
            OpenPictureCommand = new DelegateCommand(OpenPictureCommandHandler);
        }

        /// <summary>
        ///     파일 선택 창 열기
        /// </summary>
        private async void OpenPictureCommandHandler()
        {
            //FileOpePicker 설정
            var openDlg = new FileOpenPicker();
            openDlg.FileTypeFilter.Add(".jpg");
            openDlg.FileTypeFilter.Add(".jpeg");
            openDlg.FileTypeFilter.Add(".png");
            //파일 선택 창 출력
            var file = await openDlg.PickSingleFileAsync();
            if (file == null) return;
            //파일 기본 속성 조회
            var basic = await file.GetBasicPropertiesAsync();
            //4MB 이상이면 메시지 출력
            if (basic.Size > 4*1024*1024)
            {
                var msg = new MessageDialog("Please select a picture below 4MB");
                await msg.ShowAsync();
                return;
            }

            IdentifyFaceList.Clear();

            Busy.SetBusy(true, "Identifying...");

            //처음 화면에 출력용
            var bm = new BitmapImage();
            var wbm = new WriteableBitmap(100, 100);
            //IRandonAccessStream으로 파일을 열고 읽어 들이기
            using (var stream = await file.OpenAsync(FileAccessMode.Read))
            {
                await bm.SetSourceAsync(stream);
                wbm = await BitmapFactory.New(1, 1)
                    .FromStream(stream, BitmapPixelFormat.Bgra8);
            }
            BitmapSource = bm;

            //얼굴인식하고 테두리 표시하기
            //Stream으로 파일을 열어서기
            using (var stream = await file.OpenStreamForReadAsync())
            {
                //이미지에서 얼굴 탐지
                var faces = await FaceAPIHelper.Instance.GetDetectAsync(stream);
                if (faces == null) goto ExitRtn;
                //탐지된 얼굴 표시
                foreach (var face in faces)
                {
                    wbm.DrawRectangle(face.FaceRectangle.Left, face.FaceRectangle.Top,
                        face.FaceRectangle.Left + face.FaceRectangle.Width,
                        face.FaceRectangle.Top + face.FaceRectangle.Height, Colors.Red);
                }

                //한번에 인식 가능한 얼굴의 수10개로 제한
                for (var i = 0; i < faces.Length; i = i + 10)
                {
                    var faceIds = faces.Select(p => p.FaceId)
                        .Skip(i*10)
                        .Take(10)
                        .ToArray();
                    if (!faceIds.Any()) continue;
                    //인식 시작
                    var results = await FaceAPIHelper.Instance.IdentifyAsync(faceIds);
                    //결과를 루프를 돌면서 사용자 정보를 불러 옮
                    foreach (var identifyResult in results)
                    {
                        var identifyPerson = new PersonModel();
                        //탐색된 얼굴 정보 조회
                        var face = faces.First(p => p.FaceId == identifyResult.FaceId);
                        //탐색된 얼굴 이미지(빈 통)
                        var faceImage = new WriteableBitmap(face.FaceRectangle.Width, face.FaceRectangle.Height);
                        //원본 이미지에서 얼굴 부분만 복사
                        faceImage.Blit(new Rect(0, 0, faceImage.PixelWidth, faceImage.PixelHeight)
                            , wbm,
                            new Rect(face.FaceRectangle.Left, face.FaceRectangle.Top, face.FaceRectangle.Width,
                                face.FaceRectangle.Height));
                        //얼굴 추가
                        identifyPerson.AddFace(face.FaceId, faceImage);

                        if (identifyResult.Candidates.Length == 0)
                        {
                            //찾은 사람이 없음
                            identifyPerson.Name = "No one identified";
                        }
                        else
                        {
                            //가장 비슷한 사람 아이디를 반환
                            var candidate = identifyResult.Candidates.First();
                            var candidateId = candidate.PersonId;
                            //사람 정보 조회
                            var person = await FaceAPIHelper.Instance.GetPersonAsync(candidateId);

                            identifyPerson.Name = person.Name;
                            identifyPerson.PersonId = person.PersonId;
                            identifyPerson.Confidence = candidate.Confidence;

                            wbm.DrawRectangle(face.FaceRectangle.Left, face.FaceRectangle.Top,
                                face.FaceRectangle.Left + face.FaceRectangle.Width,
                                face.FaceRectangle.Top + face.FaceRectangle.Height, Colors.Blue);
                        }
                        //목록에 추가
                        IdentifyFaceList.Add(identifyPerson);
                    }
                }
            }
            BitmapSource = wbm;
            ExitRtn:
            Busy.SetBusy(false);
        }
    }
}